from leather import theme
from leather.axis import Axis
from leather.chart import Chart
from leather.data_types import Number, Text
from leather.grid import Grid
from leather.lattice import Lattice
from leather.scales import Linear, Ordinal, Scale, Temporal
from leather.series import CategorySeries, Series, key_function
from leather.shapes import Bars, Columns, Dots, Line, Shape, style_function
from leather.testcase import LeatherTestCase



import unittest


class LeatherTestCase(unittest.TestCase):
    """
    Unittest case for quickly asserting logic about charts.
    """
    def render_chart(self, chart):
        """
        Verify the column names in the given table match what is expected.
        """
        svg = chart.to_svg()

        return self.parse_svg(svg)

    def parse_svg(self, text):
        from lxml import etree

        text = text.replace(' xmlns="http://www.w3.org/2000/svg"', '').encode('utf-8')

        return etree.fromstring(text)

    def assertElementCount(self, svg, selector, count):
        series = svg.cssselect(selector)
        self.assertEqual(len(series), count)

    def assertTickLabels(self, svg, orient, compare):
        ticks = [t.text for t in svg.cssselect('.%s .tick text' % orient)]
        self.assertSequenceEqual(ticks, compare)



from leather.shapes.bars import Bars
from leather.shapes.base import Shape, style_function
from leather.shapes.columns import Columns
from leather.shapes.dots import Dots
from leather.shapes.line import Line



from decimal import Decimal

from leather.scales.base import Scale
from leather.ticks.score import ScoreTicker


class Linear(Scale):
    """
    A scale that linearly maps values from a domain to a range.

    :param domain_min:
        The minimum value of the input domain.
    :param domain_max:
        The maximum value of the input domain.
    """
    def __init__(self, domain_min, domain_max):
        if domain_min > domain_max:
            raise ValueError('Inverted domains are not currently supported.')
        elif domain_min == domain_max:
            # Default to unit scale
            self._data_min = Decimal(0)
            self._data_max = Decimal(1)
        else:
            self._data_min = Decimal(domain_min)
            self._data_max = Decimal(domain_max)

        self._ticker = ScoreTicker(self._data_min, self._data_max)

    def contains(self, v):
        """
        Return :code:`True` if a given value is contained within this scale's
        domain.
        """
        return self._data_min <= v <= self._data_max

    def project(self, value, range_min, range_max):
        """
        Project a value in this scale's domain to a target range.
        """
        value = Decimal(value)
        range_min = Decimal(range_min)
        range_max = Decimal(range_max)

        pos = (value - self._ticker.min) / (self._ticker.max - self._ticker.min)

        return ((range_max - range_min) * pos) + range_min

    def project_interval(self, value, range_min, range_max):
        """
        Project a value in this scale's domain to an interval in the target
        range. This is used for places :class:`.Bars` and :class:`.Columns`.
        """
        raise NotImplementedError

    def ticks(self):
        """
        Generate a series of ticks for this scale.
        """
        return self._ticker.ticks



import xml.etree.ElementTree as ET

from leather import theme
from leather.data_types import Text
from leather.series import CategorySeries
from leather.shapes.base import Shape
from leather.utils import X, Y


class Line(Shape):
    """
    Render a series of data as a line.

    :param stroke_color:
        The color to stroke the lines. If not provided, default chart colors
        will be used.
    :param width:
        The width of the lines. Defaults to :data:`.theme.default_line_width`.
    """
    def __init__(self, stroke_color=None, width=None, stroke_dasharray=None):
        self._stroke_color = stroke_color
        self._width = width or theme.default_line_width
        self._stroke_dasharray = stroke_dasharray or theme.default_stroke_dasharray

    def validate_series(self, series):
        """
        Verify this shape can be used to render a given series.
        """
        if isinstance(series, CategorySeries):
            raise ValueError('Line can not be used to render CategorySeries.')

        if series.data_type(X) is Text or series.data_type(Y) is Text:
            raise ValueError('Line does not support Text values.')

    def _new_path(self, stroke_color):
        """
        Start a new path.
        """
        path = ET.Element(
            'path',
            stroke=stroke_color,
            fill='none'
        )
        path.set('stroke-width', str(self._width))
        if self._stroke_dasharray != 'none':
            path.set('stroke-dasharray', self._stroke_dasharray)

        return path

    def to_svg(self, width, height, x_scale, y_scale, series, palette):
        """
        Render lines to SVG elements.
        """
        group = ET.Element('g')
        group.set('class', 'series lines')

        if self._stroke_color:
            stroke_color = self._stroke_color
        else:
            stroke_color = next(palette)

        path = self._new_path(stroke_color)
        path_d = []

        for d in series.data():
            if d.x is None or d.y is None:
                if path_d:
                    path.set('d', ' '.join(path_d))
                    group.append(path)

                path_d = []
                path = self._new_path(stroke_color)

                continue

            proj_x = x_scale.project(d.x, 0, width)
            proj_y = y_scale.project(d.y, height, 0)

            if not path_d:
                command = 'M'
            else:
                command = 'L'

            path_d.extend([
                command,
                str(proj_x),
                str(proj_y)
            ])

        if path_d:
            path.set('d', ' '.join(path_d))
            group.append(path)

        return group
